'From Squeak3.2alpha of 1 November 2001 [latest update: #4599] on 29 January 2002 at 10:47:55 am'!

!Preferences commentStamp: 'gh 1/11/2002 09:49' prior: 0!
A general mechanism to store preference choices.  The default setup treats any symbol as a potential boolean flag; flags unknown to the preference dictionary are always returned as false.  

	To open the control panel:		Preferences openFactoredPanel
	To read how to use the panel (and how to make a preference be per-project):
		 Preferences giveHelpWithPreferences

All messages are on the class side.

To query a a preference:
	Preferences logDebuggerStackToFile
or some people prefer the more verbose
	Preferences valueOfFlag: #logDebuggerStackToFile

You can make up a new preference any time.  Do not define a new message in Preferences class. Accessor methods are compiled automatically when you add a preference as illustrated below:

To add a preference (e.g. in the Postscript of a fileout):
	Preferences addPreference: #samplePreference categories: #(general browsing)
		default: true balloonHelp: 'This is an example of a preference added by a do-it'
		projectLocal: false changeInformee: nil changeSelector: nil.

To remove a preference:
	Preferences expungePreferenceNamed: #samplePreference

To change a preference programatically:
	Preferences disable: #logDebuggerStackToFile.
Or to turn it on,
	Preferences enable: #logDebuggerStackToFile.
!
]style[(220 29 81 35 893)f1,f1dPreferences openFactoredPanel;;,f1,f1dPreferences giveHelpWithPreferences;;,f1!

Object subclass: #SocketRelay
	instanceVariableNames: 'client server inPort outPort delegate log process '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Network-Kernel'!

!SocketRelay commentStamp: '<historical>' prior: 0!
A SocketRelay is an object that listens on one port using a Socket and forwards all incoming traffic to another machine and port. It "sits" inbetween and is useful for examining and debugging Socket protocols.

You can set one up for examining CVS (normally uses port 2401) by calling:

Smalltalk at: #MyRelay put: (SocketRelay forwardPort: 2041 toPort: 2041 at: 'www.bluefish.se' loggingTo: Transcript).

Right now you have to stop/start it between every session:

MyRelay start
MyRelay stop!

SocketRelay class
	instanceVariableNames: ''!

!FileDirectory methodsFor: 'file name utilities' stamp: 'gh 1/22/2002 15:45'!
lastNameFor: baseFileName extension: extension
	"Assumes a file name includes a version number encoded as '.' followed by digits 
	preceding the file extension.  Increment the version number and answer the new file name.
	If a version number is not found, set the version to 1 and answer a new file name"

	| files splits |

	files _ self fileNamesMatching: (baseFileName,'*', self class dot, extension).
	splits _ files 
			collect: [:file | self splitNameVersionExtensionFor: file]
			thenSelect: [:split | (split at: 1) = baseFileName].
	splits _ splits asSortedCollection: [:a :b | (a at: 2) < (b at: 2)].
	^splits isEmpty 
			ifTrue: [nil]
			ifFalse: [(baseFileName, '.', (splits last at: 2) asString, self class dot, extension) asFileName]! !


!ImageSegment methodsFor: 'read/write segment' stamp: 'gh 1/22/2002 15:19'!
writeForExportOn: fileStream
	"Write the segment on the disk with all info needed to reconstruct it in a new image.  For export.  Out pointers are encoded as normal objects on the disk."

	| temp |
	state = #activeCopy ifFalse: [self error: 'wrong state'].
	temp _ endMarker.
	endMarker _ nil.
	fileStream fileOutClass: nil andObject: self.
		"remember extra structures.  Note class names."
	endMarker _ temp.
! !


!SocketRelay methodsFor: 'private' stamp: 'gh 2/20/2000 20:57'!
connect
	"Create and connect sockets."

	| addr |
	server _ SimpleClientSocket new.
	addr _ NetNameResolver addressForName: delegate.
	addr ifNil: [ self error: 'Could not find address for ', delegate ].
	server connectTo: addr port: outPort.

	client _ SimpleClientSocket new.
	client listenOn: 2401.

	(server waitForConnectionUntil: Socket standardDeadline)
			ifFalse: [ self error: 'Connection failed' ].! !

!SocketRelay methodsFor: 'private' stamp: 'gh 2/22/2000 22:15'!
show: aString prefix: prefix
	"Log the traffic. Line by line using linefeed as delimiter."

	| lines |
	lines _ aString findTokens: (Array with: (Character lf)).
	lines do: [:line | log show: prefix, line; cr ]! !

!SocketRelay methodsFor: 'control' stamp: 'gh 2/20/2000 21:00'!
start
	"Start the relay."

	| buf bytes |
	self connect.
	process _ [
		buf _ String new: 50000.
		[server isConnected] whileTrue: [
			"poll the client"
			client dataAvailable ifTrue: [
				bytes _ client receiveDataInto: buf.
				self show: (buf copyFrom: 1 to: bytes) prefix: 'C:'.
				"send it to server"
				server sendData: buf count: bytes].

			"poll the server"
			server dataAvailable ifTrue: [
				bytes _ server receiveDataInto: buf.
				self show: (buf copyFrom: 1 to: bytes) prefix: 'S:'.
				"send it to client"
				client sendData: buf count: bytes ].
			(Delay forMilliseconds: 200) wait.
		]
	] fork! !

!SocketRelay methodsFor: 'control' stamp: 'gh 2/20/2000 20:08'!
stop
	"Stops the relay."

	process ifNotNil: [ process terminate ].
	process _ nil.
	client ifNotNil: [ client closeAndDestroy ].
	server ifNotNil: [ server closeAndDestroy ]
	! !

!SocketRelay methodsFor: 'accessing' stamp: 'gh 2/22/2000 22:16'!
delegate: aMachine
	"Set the delegate, the machine that we act as proxy for."

	delegate _ aMachine! !

!SocketRelay methodsFor: 'accessing' stamp: 'gh 2/19/2000 18:41'!
inPort: aPortNo
	"Set the port number."

	inPort _ aPortNo! !

!SocketRelay methodsFor: 'accessing' stamp: 'gh 2/22/2000 23:29'!
log: transcriptOrSomething
	"Set where we log traffic. It must support #show: and #cr,
	as Transcript does."

	log _ transcriptOrSomething! !

!SocketRelay methodsFor: 'accessing' stamp: 'gh 2/19/2000 18:41'!
outPort: aPortNo
	"Set the port number."

	outPort _ aPortNo! !


!SocketRelay class methodsFor: 'instance creation' stamp: 'gh 2/22/2000 22:15'!
forwardPort: inPort toPort: outPort at: delegate loggingTo: transcriptOrSomething 
	"Creates a SocketRelay."

	^ self new inPort: inPort;
	 outPort: outPort;
	 delegate: delegate;
	 log: transcriptOrSomething;
	 yourself! !


!String methodsFor: 'accessing' stamp: 'gh 3/13/2000 22:23'!
findDelimitersBackwards: delimiters startingAt: start 
	"Answer the index of the character within the receiver, starting at start, that matches one of the delimiters. If the receiver does not contain any of the delimiters, answer 0."

	start to: 1 by: -1 do: [:i |
		delimiters do: [:delim | delim = (self at: i) ifTrue: [^ i]]].
	^0! !

!String methodsFor: 'converting' stamp: 'gh 1/28/2002 16:39'!
asHtmlSource
	"Do the basic character conversion for HTML.  Leave all original return 
	and tabs in place, so can conver back by simply removing bracked 
	things. 4/4/96 tk"
	| temp |
	temp _ self copyReplaceAll: '&' with: '&amp;'.
	HtmlEntities keysAndValuesDo:
		[:entity :char |
		char = $& ifFalse:
			[temp _ temp copyReplaceAll: char asString with: '&' , entity , ';']].
	^'<pre>', temp, '</pre>'

"
	'A<&>B' asHtml
"! !

!String methodsFor: 'Celeste' stamp: 'gh 11/8/2000 20:50'!
withCRLFs
	"Return a copy of the receiver in which backslash (\) characters have been replaced with <CR><LF>. Yep, pretty ugly..."

	| stream crlf |
	crlf _ String with: Character cr with: Character lf.
	stream _ WriteStream on: String new.
	self do: [ :c | c = $\ ifTrue: [ stream nextPutAll: crlf ] ifFalse: [ stream nextPut: c ]].
	^stream contents! !

!String methodsFor: 'Celeste' stamp: 'gh 1/7/2002 09:39'!
withLFs
	"Return a copy of the receiver in which backslash (\) characters have been replaced with <LF>. Yep, pretty ugly..."

	^self replaceAll: $\ with: Character lf! !


!WriteStream methodsFor: 'character writing' stamp: 'gh 1/3/2002 16:45'!
crlf
	self nextPut: Character cr; nextPut: Character lf! !

!WriteStream methodsFor: 'character writing' stamp: 'gh 1/4/2002 14:08'!
lf
	"Append a linefeed character to the receiver."

	self nextPut: Character lf! !


!TranscriptStream methodsFor: 'access' stamp: 'gh 2/24/2000 20:14'!
characterLimit
	"Tell the views how much to retain on screen"
	^ 2000000! !

